package com.sun.jmx.remote.security;

import java.util.Collections;
import java.util.Arrays;
import java.util.Iterator;
import java.security.AccessController;
import java.security.Permission;
import java.util.Collection;
import java.security.PrivilegedAction;
import javax.management.remote.SubjectDelegationPermission;
import java.security.Principal;
import java.util.ArrayList;
import javax.security.auth.Subject;
import java.security.AccessControlContext;

public class SubjectDelegator
{
    public SubjectDelegator() {
    }
    
    public AccessControlContext delegatedContext(AccessControlContext context, Subject subject, boolean b) throws SecurityException {
        if (System.getSecurityManager() != null && context == null) {
            throw new SecurityException("Illegal AccessControlContext: null");
        }
        Collection<Principal> subjectPrincipals = getSubjectPrincipals(subject);
        final ArrayList<SubjectDelegationPermission> list = new ArrayList(subjectPrincipals.size());
        for (Principal principal : subjectPrincipals) {
            list.add(new SubjectDelegationPermission(principal.getClass().getName() + "." + principal.getName()));
        }
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                Iterator<Object> iterator = list.iterator();
                while (iterator.hasNext()) {
                    AccessController.checkPermission((Permission)iterator.next());
                }
                return null;
            }
        }, context);
        return this.getDelegatedAcc(subject, b);
    }
    
    private AccessControlContext getDelegatedAcc(Subject subject, boolean b) {
        if (b) {
            return JMXSubjectDomainCombiner.getDomainCombinerContext(subject);
        }
        return JMXSubjectDomainCombiner.getContext(subject);
    }
    
    public static synchronized boolean checkRemoveCallerContext(Subject subject) {
        try {
            for (Principal principal : getSubjectPrincipals(subject)) {
                AccessController.checkPermission(new SubjectDelegationPermission(principal.getClass().getName() + "." + principal.getName()));
            }
        }
        catch (SecurityException ex) {
            return false;
        }
        return true;
    }
    
    private static Collection<Principal> getSubjectPrincipals(Subject subject) {
        if (subject.isReadOnly()) {
            return subject.getPrincipals();
        }
        return Collections.unmodifiableList(Arrays.asList(subject.getPrincipals().toArray(new Principal[0])));
    }
}
